@@ -4,9 +4,9 @@ from .models import Station, StationList
 from .utils import _build_url, station_xml_strip, station_json_strip
 
 
-def _handle_url_action_xml(url: str):
-    stations: List[Station] = list()
-    response = shoutcast_request.call_api_xml(url)
+def _handle_url_action_xml(endpoint: str, params, session):
+    stations = list()
+    response = shoutcast_request.call_api_xml(endpoint, params=params, session=session)
     api_station_list = response.get('stationlist')
 
     if not api_station_list.get('station'):
@@ -22,9 +22,9 @@ def _handle_url_action_xml(url: str):
     return StationList(tunein=shoutcast_request.tuneins, stations=stations)
 
 
-def _handle_url_action_json(url: str) -> StationList:
-    stations: List[Station] = list()
-    response = shoutcast_request.call_api_json(url)
+def _handle_url_action_json(endpoint: str, params, session) -> StationList:
+    stations = list()
+    response = shoutcast_request.call_api_json(endpoint, params=params, session=session)
     api_station_list = response.get('stationlist')
 
     if not api_station_list.get('station'):
@@ -40,7 +40,7 @@ def _handle_url_action_json(url: str) -> StationList:
     return StationList(tunein=shoutcast_request.tuneins, stations=stations)
 
 
-def get_top_500(k: AnyStr, limit: (int, Tuple) = None, **kwargs) -> StationList:
+def get_top_500(k: AnyStr, limit: (int, Tuple) = None, session=None, **kwargs) -> StationList:
     """
     gets top 500 stations from shoutcast api
     :param k: API Dev Key.
@@ -49,16 +49,18 @@ def get_top_500(k: AnyStr, limit: (int, Tuple) = None, **kwargs) -> StationList:
     :param kwargs:
         br - Filter the stations based on bitrate specified.
         mt - Filter the stations based on media type specified, MP3 = audio/mpeg and AAC+ = audio/aacp
+    :param session: request_cache session if available
     :return: list of stations
     """
 
-    url = f'/legacy/Top500?k={k}'
-    url += _build_url(limit=limit, **kwargs)
+    endpoint = '/legacy/Top500'
+    params = dict(k=k)
+    params.update(**_build_url(limit=limit, **kwargs))
 
-    return _handle_url_action_xml(url)
+    return _handle_url_action_xml(endpoint, session=session, params=params)
 
 
-def get_stations_keywords(k, search: str, limit: (int, Tuple) = None, **kwargs) -> StationList:
+def get_stations_keywords(k, search: str, limit: (int, Tuple) = None, session=None, **kwargs) -> StationList:
     """
         Get stations which match the keyword searched on SHOUTcast Radio Directory.
        :param search: Specify the query to search
@@ -68,18 +70,20 @@ def get_stations_keywords(k, search: str, limit: (int, Tuple) = None, **kwargs)
        :param kwargs:
            br - Filter the stations based on bitrate specified.
            mt - Filter the stations based on media type specified, MP3 = audio/mpeg and AAC+ = audio/aacp
+       :param session: request_cache session if available
        :return: `List[Stations]`
     """
     if not search:
         raise Exception('Search query is required')
 
-    url = f"legacy/stationsearch?k={k}&search={search.replace(' ', '+').strip()}"
-    url += _build_url(limit, **kwargs)
+    endpoint = "/legacy/stationsearch"
+    params = dict(k=k, search=search)
+    params.update(**_build_url(limit, **kwargs))
 
-    return _handle_url_action_xml(url)
+    return _handle_url_action_xml(endpoint, session=session, params=params)
 
 
-def get_stations_by_genre(k, genre: str, limit: (int, Tuple) = None, **kwargs) -> StationList:
+def get_stations_by_genre(k, genre: str, limit: (int, Tuple) = None, session=None, **kwargs) -> StationList:
     """
        Get stations which match the genre specified as query.
        :param genre: genre
@@ -89,18 +93,20 @@ def get_stations_by_genre(k, genre: str, limit: (int, Tuple) = None, **kwargs) -
        :param kwargs:
            br - Filter the stations based on bitrate specified.
            mt - Filter the stations based on media type specified, MP3 = audio/mpeg and AAC+ = audio/aacp
+       :param session: request_cache session if available
        :return: `List[Stations]`
     """
     if not genre:
         raise Exception('genre is required')
 
-    url = f"legacy/stationsearch?k={k}&search={genre.replace(' ', '+').strip()}"
-    url += _build_url(limit, **kwargs)
+    endpoint = "/legacy/stationsearch"
+    params = dict(k=k, search=genre)
+    params.update(**_build_url(limit, **kwargs))
 
-    return _handle_url_action_xml(url)
+    return _handle_url_action_xml(endpoint, session=session, params=params)
 
 
-def get_stations_by_now_playing(k, ct: str, limit: (int, Tuple) = None, **kwargs) -> StationList:
+def get_stations_by_now_playing(k, ct: str, limit: (int, Tuple) = None, session=None, **kwargs) -> StationList:
     """
        Return stations which match a specified query in the now playing node.
        :param ct: Query to search in Now Playing node. This parameter also supports querying multiple artists in the same query by using "||". ex: ct=madonna||u2||beyonce up to 10 artists
@@ -110,19 +116,21 @@ def get_stations_by_now_playing(k, ct: str, limit: (int, Tuple) = None, **kwargs
        :param kwargs:
            br - Filter the stations based on bitrate specified.
            mt - Filter the stations based on media type specified, MP3 = audio/mpeg and AAC+ = audio/aacp
+       :param session: request_cache session if available
        :return: `List[Stations]`
     """
     if not ct:
         raise Exception('genre is required')
 
-    url = f"station/nowplaying?k={k}&ct={ct.replace(' ', '+').strip()}&f=json"
-    url += _build_url(limit, **kwargs)
+    endpoint = "/station/nowplaying"
+    params = dict(k=k, f='json', ct=ct)
+    params.update(**_build_url(limit, **kwargs))
 
-    return _handle_url_action_json(url)
+    return _handle_url_action_json(endpoint, session=session, params=params)
 
 
 def get_stations_bitrate_or_genre_id(k, br: int = 128,
-                                     genre_id: int = None, limit: (int, Tuple) = None, **kwargs) -> StationList:
+                                     genre_id: int = None, limit: (int, Tuple) = None, session=None, **kwargs) -> StationList:
     """
           Get stations which match the genre specified as query.
           :param genre_id: genre id
@@ -132,19 +140,21 @@ def get_stations_bitrate_or_genre_id(k, br: int = 128,
           `limit=(X,Y)` - Y is the number of results to return and X is the offset.
           :param kwargs:
               mt - Filter the stations based on media type specified, MP3 = audio/mpeg and AAC+ = audio/aacp
+          :param session: request_cache session if available
           :return: `List[Stations]`
        """
 
     if not br and not genre_id:
         raise Exception('genre_id or br is required')
 
-    url = f"station/advancedsearch?k={k}&f=json"
-    url += _build_url(limit, br=br, genre_id=genre_id, **kwargs)
+    endpoint = "/station/advancedsearch"
+    params = dict(k=k, f='json')
+    params.update(**_build_url(limit, br=br, genre_id=genre_id, **kwargs))
 
-    return _handle_url_action_json(url)
+    return _handle_url_action_json(endpoint, session=session, params=params)
 
 
-def get_random_station(k, limit: (int, Tuple) = None, **kwargs):
+def get_random_station(k, limit: (int, Tuple) = None, session=None, **kwargs):
     """
           Get random stations on SHOUTcast Radio Directory. Random stations can be restricted
           to the Bitrate/Genre/Media type specified.
@@ -155,10 +165,12 @@ def get_random_station(k, limit: (int, Tuple) = None, **kwargs):
               br - Filter the stations based on bitrate specified.
               mt - Filter the stations based on media type specified, MP3 = audio/mpeg and AAC+ = audio/aacp
               genre - Genre to filter the station result.
+          :param session: request_cache session if available
           :return: `List[Stations]`
        """
 
-    url = f"station/randomstations?k={k}&f=json"
-    url += _build_url(limit, **kwargs)
+    endpoint = "/station/randomstations"
+    params = dict(k=k, f='json')
+    params.update(**_build_url(limit, **kwargs))
 
-    return _handle_url_action_json(url)
+    return _handle_url_action_json(endpoint, session=session, params=params)
