@@ -1,26 +1,27 @@
+import requests
 from typing import Tuple
 
 
-def _build_url(limit: (int, Tuple), **kwargs) -> str:
-    url = ""
+def _build_url(limit: (int, Tuple), **kwargs) -> dict:
+    url_params = dict()
     if isinstance(limit, tuple):
         x, y = limit
-        url += f'&limit={x},{y}'
+        url_params.update(limit='{},{}'.format(x, y))
     elif limit:
-        url += f'&limit={limit}'
+        url_params.update(limit=str(limit))
 
     if kwargs.get('br'):
-        url += f"&br={kwargs.get('br')}"
+        url_params.update(br=kwargs.get('br'))
 
     if kwargs.get('mt'):
-        url += f"&mt={kwargs.get('mt')}"
+        url_params.update(mt=kwargs.get('mt'))
 
     if kwargs.get('genre_id'):
-        url += f"&genre_id={int(kwargs.get('genre_id'))}"
+        url_params.update(genre_id=kwargs.get('genre_id'))
 
     if kwargs.get('genre'):
-        url += f"&genre={int(kwargs.get('genre'))}"
-    return url
+        url_params.update(genre=kwargs.get('genre'))
+    return url_params
 
 
 def station_xml_strip(station):
@@ -46,20 +47,20 @@ def station_json_strip(station):
 def _get_all_genre(station):
     genre = station.get('@genre')
     for i in range(2, 10):
-        genre_ = station.get(f'@genre{i}')
+        genre_ = station.get('@genre{}'.format(i))
         if not genre_:
             return genre
-        genre += f", {genre_}"
+        genre += ", {}".format(genre_)
     return genre
 
 
 def _get_all_genre_from_json(station):
     genre = station.get('genre')
     for i in range(2, 10):
-        genre_ = station.get(f'genre{i}')
+        genre_ = station.get('genre{}'.format(i))
         if not genre_:
             return genre
-        genre += f", {genre_}"
+        genre += ", {}".format(genre_)
     return genre
 
 
@@ -68,3 +69,9 @@ def genre_xml_strip(genre):
     item['name'] = genre.get('@name')
     item['count'] = int(genre.get('@count'))
     return item
+
+
+def _init_session(session):
+    if session is None:
+        session = requests.Session()
+    return session
